(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.remUtils = factory());
}(this, (function () { 'use strict';

  /**
   * Created by:tm  Date:2018/11/8
   */
  /**
   * 获取url参数
   * @param url
   * @param name
   * @returns {*}
   */
  var getUrlSearch = function getUrlSearch(url, name) {
    var reg = new RegExp('(^|&)' + name + '=([^&]*)(&|$)', 'i');
    var r = url.slice(url.indexOf('?') + 1).match(reg);
    if (r != null) {
      try {
        return decodeURIComponent(r[2]);
      } catch (_e) {
        return null;
      }
    }
    return null;
  };

  /**
   * Created by:tm  Date:2018/9/4
   */
  /**
   * @desc 函数防抖
   * @param func 函数
   * @param wait 延迟执行毫秒数
   * @param immediate true 表立即执行，false 表非立即执行
   */
  function debounce(func, wait, immediate) {
    var timeout = void 0;

    return function () {
      var context = this;
      // args = [].slice.call(arguments)
      var args = arguments;

      if (timeout) clearTimeout(timeout);
      if (immediate) {
        var callNow = !timeout;
        timeout = setTimeout(function () {
          timeout = null;
        }, wait);
        if (callNow) func.apply(context, args);
      } else {
        timeout = setTimeout(function () {
          func.apply(context, args);
        }, wait);
      }
    };
  }

  /**
   * Created by:tm  Date:2018/9/7
   */
  /**
   * @desc 函数节流
   * @param func 函数
   * @param wait 延迟执行毫秒数
   */
  var throttle = function throttle(func, wait) {
    var timeout = void 0;
    return function () {
      var context = this;
      // args = [].slice.call(arguments)
      var args = arguments;

      if (!timeout) {
        timeout = setTimeout(function () {
          timeout = null;
          func.apply(context, args);
        }, wait);
      }
    };
  };

  /**
   * Created by:tm  Date:2018/9/18
   */
  /**
   * 获取 localStorage 存储的数据，如果有缓存时间，就根据时间判断，没有直接返回
   * @param key
   * @param def
   * @returns {any}
   */

  var getLocalItem = function getLocalItem(key) {
    var def = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var o = localStorage.getItem(key);
    if (!o) {
      return def;
    }
    // || (o._t && o._t < +new Date())
    if (/"_t":/.test(o)) {
      var _JSON$parse = JSON.parse(o),
          _t = _JSON$parse._t,
          _d = _JSON$parse._d;

      console.log(key, '缓存时间', new Date(_t), '过期', _t < +new Date());
      if (_t < +new Date()) {
        console.log('缓存过期');
        return def;
      } else {
        console.log('使用缓存');
        return _d;
      }
    } else {
      console.log('没有缓存时间');
      return JSON.parse(o || JSON.stringify(def));
    }
  };

  /**
   * Created by:tm  Date:2018/9/18
   */
  /**
   * 存储数据到loaclStorage 对于item才给定过期时间 半小时计算
   * @param key
   * @param data
   * @param expires
   */
  var setLocalItem = function setLocalItem(key, data) {
    var expires = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 72;

    var o = {
      _t: expires * 18e5 + new Date().getTime(),
      _d: data
    };
    localStorage.setItem(key, JSON.stringify(o));
  };

  /**
   * Created by:tm  Date:2018/9/18
   */
  var RULES = {
    // ANDROID: /(^| |\()(android)(;| |\/|$)/i,
    ANDROID: /Android/i,
    IOS: /iPhone|iPad|iPod/i,
    // IOS: /(^| |\()(iPhone|iPad|iPod)(;| |\/|$)/i,
    WEIXIN: /(^| )MicroMessenger( |\/|$)/i,
    QQ: /(^| )(QQ)( |\/|$)/i,
    QQBROWSER_HD: /iPad.*?(^| )(MQQBrowser)( |\/|$)/i,
    QQBROWSER: /(^| )(MQQBrowser)( |\/|$)/i
  };

  var clientHelper = function clientHelper(conf) {
    var UA = window.navigator.userAgent;
    var isIOS = RULES.IOS.test(UA);
    var isAndroid = RULES.ANDROID.test(UA);
    var isQQ = RULES.QQ.test(UA);
    var isWX = RULES.WEIXIN.test(UA);
    var os = isAndroid ? UA.match(RULES.ANDROID)[0].toLowerCase() : isIOS ? UA.match(RULES.IOS)[0].toLowerCase() : 'android';
    var isApp = false;
    if (conf) {
      var app = UA.match(conf.reg);
      if (app) {
        isApp = app[0].toLowerCase();
      }
    }
    return {
      os: os,
      isApp: isApp,
      isIOS: isIOS,
      isAndroid: isAndroid,
      isQQ: isQQ,
      isWX: isWX
    };
  };

  /**
   * Created by:tm  Date:2018/10/11
   */
  /**
   * 如果用户已经安装了应用 抛出schemes,打开app对应页面
   * @param src
   * @param options
   * @private
   */
  var createIframe = function createIframe(src) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var ifm = document.getElementById(options.id);
    if (ifm) {
      ifm.src = src;
      return;
    }
    var ifr = document.createElement('iframe');
    ifr.style.display = 'none';
    ifr.src = src;
    document.body.appendChild(ifr);
  };

  /**
   * Created by:tm  Date:2018/10/11
   */
  /**
   * 配置跳转相关字段
   * @param data
   * @returns {*}
   * @private
   */
  var getScheme = function getScheme(data) {
    var scheme_Adr = void 0;
    var url = void 0;
    var iosUrl = void 0;
    if (data.scheme_Adr) {
      // 配置跳转规则
      if (data.webView) {
        scheme_Adr = data.scheme_Adr + '://goto?page=web&url=' + encodeURIComponent(location.href);
      } else {
        var comicId = data.comic_id;
        scheme_Adr = data.scheme_Adr + '://goto?page=' + (comicId ? 'dir' : 'index') + '&comic_id=' + comicId;
      }
    }
    // 生成APP下载地址
    if (data.link) {
      url = data.link;
      iosUrl = data.link;
    } else {
      url = 'http://a.app.qq.com/o/simple.jsp?pkgname=' + data.pkgname + '&ckey=' + data.ckey;
      iosUrl = data.iosurl || url;
    }
    return Object.assign({}, data, {
      url: url,
      iosUrl: iosUrl,
      scheme_Adr: scheme_Adr
    });
  };

  /**
   * Created by:tm  Date:2018/10/11
   */

  /**
   * 调用app方法
   * @param config
   * @returns {boolean}
   * @private
   */
  var schemeHandle = function schemeHandle(config) {
    var UA = navigator.userAgent.toLowerCase();
    var schemeAdr = config.scheme_Adr;
    // 没有schemeAdr 只做下载页跳转，进行导流
    if (!schemeAdr) {
      if (UA.match(/(iPhone|iPod|iPad);?/i)) {
        window.location.href = config.iosUrl || config.url;
      } else {
        window.location.href = config.url;
      }
      return false;
    }
    // app内，直接执行方法
    if (UA.match(/(zymk|kmh|mht|aym|smh|asmh);?/i)) {
      window.location.href = schemeAdr;
      return false;
    }
    if (UA.match(/(iPhone|iPod|iPad);?/i)) {
      // 抛出schemes,打开app对应页面
      if (/MicroMessenger/i.test(UA) || /(^| )(QQ)( |\/|$)/i.test(UA)) {
        window.location.href = config.iosUrl ? config.iosUrl + '?' + decodeURIComponent(schemeAdr.split('?')[1]) : config.url;
      } else {
        window.location.href = schemeAdr;
        // 由于部分ios中打开app后，WAP页面会被挂起，定时器不会被执行。因此可以做下优化：
        // WAP页重新被聚焦后，如果超过1s，认为APP被打开了，重新聚焦时就不必再跳转到APP下载页
        var startTime = Date.now();
        var delayTime = /safari/.test(UA) ? 3000 : 1000;
        var timer = setInterval(function () {
          var endTime = Date.now();
          if (endTime - startTime > delayTime) {
            clearInterval(timer);
            window.location.href = config.iosUrl || config.url;
          }
        }, 25);
      }
    } else {
      if (/MicroMessenger/i.test(UA)) {
        window.location.href = config.iosUrl ? config.iosUrl + '?' + decodeURIComponent(schemeAdr.split('?')[1]) : config.url;
      } else {
        createIframe(schemeAdr);
        // 延时跳转下载页
        setTimeout(function () {
          window.location.href = config.url;
        }, 600);
      }
    }
  };

  /**
   * Created by:tm  Date:2018/9/4
   */

  // polly
  if (typeof Object.assign !== 'function') {
    // Must be writable: true, enumerable: false, configurable: true
    Object.defineProperty(Object, "assign", {
      value: function assign(target, varArgs) {

        if (target == null) {
          // TypeError if undefined or null
          throw new TypeError('Cannot convert undefined or null to object');
        }

        var to = Object(target);

        for (var index = 1; index < arguments.length; index++) {
          var nextSource = arguments[index];

          if (nextSource != null) {
            // Skip over if undefined or null
            for (var nextKey in nextSource) {
              // Avoid bugs when hasOwnProperty is shadowed
              if (Object.prototype.hasOwnProperty.call(nextSource, nextKey)) {
                to[nextKey] = nextSource[nextKey];
              }
            }
          }
        }
        return to;
      },
      writable: true,
      configurable: true
    });
  }

  /**
   * 判断空对象
   */
  var isEmptyObject = function isEmptyObject(e) {
    var t = void 0;
    for (t in e) {
      if (e.hasOwnProperty(t)) return !1;
    }return !0;
  };

  var index = {
    // 获取url参数
    getUrlSearch: getUrlSearch,
    // 判断空对象
    isEmptyObject: isEmptyObject,
    // 防抖
    debounce: debounce,
    // 节流
    throttle: throttle,
    // 获取local
    getLocalItem: getLocalItem,
    // 存储local
    setLocalItem: setLocalItem,
    // 客户端信息
    clientHelper: clientHelper,
    // 创建iframe
    createIframe: createIframe,
    // 配置scheme字段
    getScheme: getScheme,
    // 调起app跳转
    schemeHandle: schemeHandle
  };

  return index;

})));
